@@ -188,6 +188,41 @@ public abstract class Page implements Serializable {
 	private String	doctype_def = null;
 	private static final long serialVersionUID = 2L;
 
+	private Popup.Type popup_type = null;
+
+	private static volatile short page_id;
+	private static final Object	obj4uniKey = new Object();
+	private String uniKey;
+
+	private void makeKey() {
+		long	now = 0;
+		synchronized( obj4uniKey ) {
+			now = page_id;
+			page_id++;
+		}
+		now = (now << 56) ^ (new Date().getTime() & 0x00ffffffffffffffL);
+		now &= 0x7fffffffffffffffL;
+		StringBuilder	buf = new StringBuilder( "paraselene_page_key_" );
+		buf = buf.append( Long.toString( now, Character.MAX_RADIX ) );
+		uniKey = buf.toString();
+	}
+
+	/**
+	 * ポップアップ状態を返します。
+	 * @return ポップアップ状態。
+	 * ポップアップでなければ null です。
+	 */
+	public Popup.Type getPopupType() {
+		return popup_type;
+	}
+
+	/**
+	 * 使用しないで下さい。
+	 */
+	public void setPopupType( Popup.Type type ) {
+		popup_type = type;
+	}
+
 	/**
 	 * 実行状態であるか？常に true を返します。
 	 * @return true:実行中、false:モックアップ解析中。
@@ -552,6 +587,7 @@ public abstract class Page implements Serializable {
 	 * コンストラクタ。
 	 */
 	public Page() {
+		makeKey();
 	}
 
 	/**
@@ -600,7 +636,7 @@ public abstract class Page implements Serializable {
 	 * @return ユニークキー。
 	 */
 	public String getUniqueKey() {
-		return Integer.toString( super.hashCode() );
+		return uniKey;
 	}
 
 	/**
@@ -694,6 +730,7 @@ public abstract class Page implements Serializable {
 		String	name = tag.getNameAttribute();
 		if ( name != null ) {
 			if ( Form.FORM_ID.equals( name ) )	return;
+			if ( Form.FORM_AJAX.equals( name ) )	return;
 		}
 		nameTagSingle( tag );
 		int	cnt = tag.getHTMLPartCount();
@@ -741,12 +778,13 @@ public abstract class Page implements Serializable {
 	}
 
 	private void clearAssignedPage( Tag tag ) {
-		tag.setAssignedPage( null );
 		Tag[]	next = tag.getTagArray();
-		if ( next == null )	return;
-		for ( int i = 0; i < next.length; i++ ) {
-			clearAssignedPage( next[i] );
+		if ( next != null ) {
+			for ( int i = 0; i < next.length; i++ ) {
+				clearAssignedPage( next[i] );
+			}
 		}
+		removeNameEntry( tag );
 	}
 
 	/**
@@ -1009,33 +1047,48 @@ public abstract class Page implements Serializable {
 				}
 			}
 			else	ajax_f = false;
-			if ( body != null && ajax != AjaxSupport.NO ) {
+			if ( head != null && body != null && ajax != AjaxSupport.NO && getPopupType() == null ) {
+				Tag	script = new Tag( "script", false );
+				try {
+					script.setAttribute(
+						new Attribute( "type", "text/javascript" ),
+						new Attribute( "src", new URIValue( "paraselene.js.na" ) )
+					);
+				}
+				catch( Exception e ) {
+					Option.debug( e );
+				}
+				head.addHTMLPart( script );
+
+				Tag[]	div = new Tag[] { new Tag( "div", false ), new Tag( "div", false ) };
+				div[0].include( body );
+				Tag eclipse = new Tag( "div", false );
+				eclipse.setAttribute(
+					new Attribute( "id", AJAX_BLACK_ID )
+				);
 				Tag	iframe = new Tag( "iframe", false );
 				iframe.setAttribute(
 					new Attribute( "id", AJAX_TARGET_ID ),
-					new Attribute( "height", new NumberValue( 1 ) ),
-					new Attribute( "width", new NumberValue( 1 ) ),
+					new Attribute( "name", AJAX_TARGET_ID ),
 					new Attribute( "scrolling", new Text( "no" ) ),
-					new Attribute( "style",
-						new Style(
-							new Property( "position", new Text( "absolute" ) ),
-							new Property( "top", new NumberValue( -100 ) ),
-							new Property( "left", new NumberValue( -100 ) ),
-							new Property( "overflow", new Text( "hidden" ) ),
-							new Property( "visibility", new Text( "hidden" ) )
-						)
-					)
+					new Attribute( "onload", "paraselene_form_comp();" )
 				);
-				Tag	unique = new Tag( "script", false );
-				unique.addHTMLPart( new PlainText( String.format( "var page_id=%s", getUniqueKey() ) ) );
-				body.addHTMLPart( iframe, unique );
+				div[1].addHTMLPart( eclipse, iframe );
+				div[1].setAttribute(
+					new Attribute( "id", AJAX_ADD_ID )
+				);
+				body.removeHTMLPart();
+				body.addHTMLPart( div[0],div[1] );
 			}
 			else	ajax_f = false;
 		}
 		else	ajax_f = false;
+		if ( getPopupType() != null )	ajax_f = true;
 	}
 
-	public static final String	AJAX_TARGET_ID = "paraselene$Dragon'sTail";
+	private static final String	AJAX_BLACK_ID = "paraseleneEclipse";
+	private static final String	AJAX_TARGET_ID = "paraseleneDragonsTail";
+	private static final String	AJAX_ADD_ID = "paraseleneDragonsHead";
 	private boolean ajax_f = false;
 	private transient RequestParameter	now_req;
 	private transient RequestParameter.Mobile			access_mobile;
@@ -1071,5 +1124,29 @@ public abstract class Page implements Serializable {
 	public RequestParameter.SearchEngine getAccessedSearchEngine() {
 		return access_se;
 	}
+
+	public Tag[] getModifiedTag() {
+		Tag	tag = getFirstTagByType( "body" );
+		if ( tag == null )	return new Tag[0];
+		return tag.getModifiedTag();
+	}
+
+	/**
+	 * Ajax 使用時の Z インデックスのベースを返します。
+	 * 必要に応じてオーバーライドして下さい。
+	 * @return 常に 1 を返します。
+	 */
+	public int getZindex() {
+		return 1;
+	}
+
+	/**
+	 * ポップアップの背景スタイルを指定します。
+	 * 必要に応じてオーバーライドして下さい。
+	 * @return 常に lavender 指定を返します。
+	 */
+	public CSSValuable[] getPopupBackGround() {
+		return new CSSValuable[] { new Color( WebColor.LAVENDER ) };
+	}
 }
 
