@@ -18,18 +18,37 @@ class HistorySet implements Serializable {
 	static final int	RAND = 0x7fffffff;
 	private HashMap<String,Integer>		set_no = new HashMap<String,Integer>();
 	private HashMap<Integer,History>	hist = new HashMap<Integer,History>();
+	private String session_id;
 
-	HistorySet() {
+	HistorySet( String sid) {
+		session_id = sid;
 		newNo( ROOT );
 	}
 
+	synchronized int getKey( String name ) {
+		Integer	key = set_no.get( name );
+		if ( key == null )	return -1;
+		return key;
+	}
+
 	synchronized History get( int key ) {
 		return hist.get( key );
 	}
 
 	synchronized void remove( int key ) {
 		if ( set_no.containsValue( new Integer( key ) ) )	return;
-		hist.remove( key );
+		History	h = hist.remove( key );
+		if ( h != null ) {
+			h.drop_f = true;
+			h.removePage();
+		}
+	}
+
+	synchronized void drop() {
+		set_no.clear();
+		for ( int key: hist.keySet() ) {
+			remove( key );
+		}
 	}
 
 	synchronized private int newNo( int no ) {
@@ -39,7 +58,7 @@ class HistorySet implements Serializable {
 				if ( no < 0 )	no = 1;
 				continue;
 			}
-			hist.put( no, new History( no ) );
+			hist.put( no, new History( no, session_id ) );
 			return no;
 		}
 	}
