@@ -34,7 +34,7 @@ class DebugEcho implements HttpSessionBindingListener {
 		if ( map.get( map_key ) != null )	return;
 		map.put( map_key, this );
 		session.setAttribute( SESSION_KEY + key, this );
-		Ajax.add( session, key, null );
+		Ajax.add( session, key );
 	}
 
 	boolean isSame( HttpSession session, String k ) {
