@@ -6,13 +6,15 @@
  */
 package paraselene.supervisor;
 
+import paraselene.*;
 
 /**
  * 遷移先指定。output するページをポップアップ表示します。<br>
  * ただし、遷移元ページのgetAjaxSupportが有効でない場合は、通常の Forward と等価です。
  */
-public class Popup extends Forward {
+public class Popup extends Forward implements AjaxForward {
 	private static final long serialVersionUID = 2L;
+	Page active_page = null;
 	/**
 	 * ポップアップ種別。
 	 */
@@ -38,5 +40,34 @@ public class Popup extends Forward {
 		super( id );
 		type = dialog_type;
 	}
+
+	/**
+	 * コンストラクタ。
+	 * 指定ページの Z順を手前にします。
+	 * @param page MODELESSのポップアップ。
+	 */
+	public Popup( Page page ) {
+		this( page.getID(), Type.MODELESS );
+		active_page = page;
+	}
+
+	void overwriteType( Type t ) {
+		if ( t == Type.MODELESS )	return;
+		type = t;
+	}
+
+	Type getType() {
+		return type;
+	}
+
+	/**
+	 * 同じ動作であるか？
+	 * 常にfalseを返します。
+	 * @param o 比較対象。
+	 * @return true:同一動作、false:異なる動作。
+	 */
+	public boolean equals( Object o ) {
+		return false;
+	}
 }
 
