@@ -21,8 +21,6 @@ class DataHolder {
 	HttpServletRequest	req;
 	HttpServletResponse	response;
 	SessionData	data;
-	PageID	myself = null;
-	boolean	history_use = false;
 	boolean	session_null = true;
 	boolean search_engine = false;
 
@@ -36,10 +34,10 @@ class RedirectData implements Serializable {
 	Page[]	out_page;
 }
 
-class SessionData implements Serializable {
+class SessionData implements Serializable, HttpSessionBindingListener {
 	private static final long serialVersionUID = 2L;
 	TransactionSequencer seq = new TransactionSequencer();
-	HistorySet hist = new HistorySet();
+	HistorySet hist = null;
 	private HashMap<PageID, RedirectData>	pending = new HashMap<PageID, RedirectData>();
 	private volatile int client;
 
@@ -61,6 +59,7 @@ class SessionData implements Serializable {
 
 	SessionData( HttpServletRequest req ) {
 		client = getClient( req );
+		hist = new HistorySet( req.getSession().getId() );
 	}
 
 	boolean isSame( HttpServletRequest req ) {
@@ -83,11 +82,11 @@ class SessionData implements Serializable {
 		return true;
 	}
 
-	Forward redirect( Page[] out_ret, Forward fw, DataHolder holder ) throws Exception {
+	Forward redirect( Page[] out_ret, Forward fw, RequestParameter req ) throws Exception {
 		RedirectData	rd = new RedirectData();
 		rd.fw = fw;
 		rd.out_page = out_ret;
-		URI	uri = holder.request.getURI();
+		URI	uri = req.getURI();
 		rd.uri = TransactionSequencer.makeRedirectURI( seq, out_ret[0].getID() );
 		String	path = URIValue.makeAbsolutePath(
 			URIValue.Scheme.getScheme( uri ), uri.getPort(), rd.uri
@@ -97,6 +96,11 @@ class SessionData implements Serializable {
 		}
 		return new Forward( new URI( path ), false );
 	}
+
+	public void valueBound( HttpSessionBindingEvent ev ) {}
+	public void valueUnbound( HttpSessionBindingEvent ev ) {
+		hist.drop();
+	}
 }
 
 
