@@ -10,6 +10,7 @@ package paraselene.supervisor;
 import paraselene.*;
 import paraselene.ajax.*;
 import paraselene.tag.*;
+import paraselene.tag.form.*;
 import java.io.*;
 import java.net.*;
 import java.util.*;
@@ -73,7 +74,7 @@ public abstract class Supervisor extends HttpServlet {
 		super();
 	}
 
-	private static String resolvePage( DataHolder holder, String p, int from_key, int to_key ) {
+	private static String resolvePage( DataHolder holder, PageID from_page, String p, int from_key, int to_key ) {
 		try {
 			URI	u = new URI( p );
 			PageID	to = null;
@@ -89,7 +90,7 @@ public abstract class Supervisor extends HttpServlet {
 					TransactionSequencer.makeURI(
 						( page.isCheckRepeatSameRequest() )?
 							holder.data.seq: null,
-						holder.myself, to, from_key, to_key, !holder.search_engine
+						from_page, to, from_key, to_key, !holder.search_engine
 					)
 				);
 				String	query = u.getQuery();
@@ -180,7 +181,7 @@ public abstract class Supervisor extends HttpServlet {
 			}
 			switch( def_no ) {
 			case 0:
-				part[i] = resolvePage( holder, part[i], from_key, to_key );
+				part[i] = resolvePage( holder, tag.getAssignedPage().getID(), part[i], from_key, to_key );
 				break;
 			case 1:
 			case 2:
@@ -322,7 +323,6 @@ public abstract class Supervisor extends HttpServlet {
 			}
 
 			boolean	session_f = false;
-			holder.history_use = false;
 			req = holder.request;
 			session = req.getSession();
 			String	uri = req.getURI().toString();
@@ -376,7 +376,27 @@ public abstract class Supervisor extends HttpServlet {
 			}
 			last_page = path_data.from;
 			next_page = path_data.to;
-			if ( last_page != null ) {
+			RequestItem	ajax = req.getItem( A_AJAX_KEY );
+			if ( ajax == null )	ajax = req.getItem( Form.FORM_AJAX );
+			if ( ajax != null ) {
+				String	call_key = ajax.getValue( 0 );
+				Page	sub = data.hist.get( (path_data.from_histry != null)? path_data.from_histry: HistorySet.ROOT ).getBrowsingPage( call_key );
+				if ( sub != null ) {
+					fact.returnPage( last_page );
+					last_page = sub;
+				}
+				else if ( !"-1".equals( call_key ) ) {
+					fw = onIllegalRequest( IllegalRequest.SESSION_ERROR, req, last_page, next_page );
+					if ( fw == null || fw.getStatus() == 0 ) {
+						fw = new Forward( 403 );
+					}
+					fw.Action( this, res );
+					clear_session( session );
+					return;
+				}
+				else	ajax = null;
+			}
+			if ( ajax == null && last_page != null ) {
 				if ( last_page instanceof Null ) {
 					System.out.println( "last page is null" );
 					fw = new Forward( 403 );
@@ -405,7 +425,7 @@ public abstract class Supervisor extends HttpServlet {
 					}
 				}
 			}
-			else {
+			else if ( ajax == null ) {
 				last_page = fact.getNullPage();
 			}
 			if ( next_page != null ) {
@@ -448,10 +468,15 @@ public abstract class Supervisor extends HttpServlet {
 					}
 					else {
 						if ( last_page.getID() == next_page.getID() ) {
-							fw = new Feedback( next_page.getID() );
+							fw = new Feedback( last_page );
 						}
 						else if ( Closure.isClosure( last_page, next_page, req ) ) {
-							fw = new Closure( next_page.getID() );
+							if ( last_page.getPopupType() == null ) {
+								fw = new Closure( next_page );
+							}
+							else {
+								fw = new Closure( last_page );
+							}
 						}
 						else {
 							fw = new Forward( next_page.getID(), true, false );
@@ -464,13 +489,11 @@ public abstract class Supervisor extends HttpServlet {
 				fw = SandBox.input( last_page, req, fw );
 				last_page.setInitialized( false );
 				last_page.setRequestParameter( null );
-				holder.history_use = fw.history_f;
 				session_f = fw.session_clear_f;
 				if ( fw.getPageID() == Closure.CLOSE ) {
-					fw = new Feedback( last_page.getID() );
+					fw = new Feedback( last_page );
 				}
-				next_page = fw.getPage( last_page );
-				if ( next_page == null ) {
+				if ( (!last_page.isAjax() && fw.isLeave()) || fw.getStatus() != 0 ) {
 					fw.Action( this, res );
 					if ( session_f )	if (session != null )	clear_session( session );
 					return;
@@ -480,7 +503,34 @@ public abstract class Supervisor extends HttpServlet {
 			if ( path_data.to_histry != null ) {
 				out_hist_key = path_data.to_histry;
 			}
-			Page[]	out_ret = output_call( out_hist_key, last_page, next_page );
+			OutputHolder	oh = new OutputHolder( out_hist_key, last_page, fw, req, data );
+			oh.call();
+			String	key = last_page.getUniqueKey();
+			ajax = req.getItem( A_AJAX_KEY );
+			if ( ajax != null ) {
+				String	call_key = ajax.getValue( 0 );
+				if ( key.equals( call_key ) ) {
+					Ajax.remove( session, key );
+					Ajax.add( session, key, oh.getPostBack() );
+					Page	ajax_page = getPageFactory().getPage( PAGE_ID_JSON );
+					ajax_page.output( null, req );
+					write( new Page[]{ ajax_page, ajax_page }, session_f, fw, holder );
+					return;
+				}
+			}
+			ajax = req.getItem( Form.FORM_AJAX );
+			if ( ajax != null ) {
+				String	call_key = ajax.getValue( 0 );
+				if ( key.equals( call_key ) ) {
+					Ajax.remove( session, key );
+					Ajax.add( session, key, oh.getPostBack() );
+					Page	ajax_page = getPageFactory().getPage( PAGE_ID_FORM_JSON );
+					ajax_page.output( last_page, req );
+					write( new Page[]{ ajax_page, ajax_page }, session_f, fw, holder );
+					return;
+				}
+			}
+			Page[]	out_ret = oh.getPage();
 			if ( out_ret == null ) {
 				System.out.println( "output_call false" );
 				fw = new Forward( 500 );
@@ -494,6 +544,13 @@ public abstract class Supervisor extends HttpServlet {
 		}
 	}
 
+	private static final int PAGE_ID_JSON = -1;
+	private static final int PAGE_ID_FORM_JSON = -2;
+	/**
+	 * Aタグからの AJAX 要求。
+	 */
+	public static final String A_AJAX_KEY = "paraselene_a_ajax";
+
 	HttpSession makeNewSession( HttpSession old ) {
 		DataHolder	holder = null;
 		synchronized( data_map_th ) {
@@ -531,9 +588,8 @@ public abstract class Supervisor extends HttpServlet {
 			session = holder.req.getSession( true );
 			session.setAttribute( SESSION_DATA, data );
 		}
-		holder.myself = next_page.getID();
 		if ( param.getMethod() == RequestParameter.Method.POST ) {
-			data.redirect( out_ret, fw, holder ).Action( this, res );
+			data.redirect( out_ret, fw, param ).Action( this, res );
 		}
 		else {
 			String	mime = next_page.getContentType();
@@ -690,46 +746,6 @@ public abstract class Supervisor extends HttpServlet {
 		return true;
 	}
 
-	private Page[] output_call( int next_key, Page last_page, Page next_page ) throws Throwable {
-		next_page.setHistoryKey( next_key );
-		PageID	last_id = null;
-		if ( last_page instanceof NullPage ) {
-			last_page = null;
-		}
-		if ( last_page != null ) {
-			last_id = last_page.getID();
-		}
-		DataHolder	holder = null;
-		SessionData	data = null;
-		synchronized( data_map_th ) {
-			holder = data_map_th.get( Thread.currentThread() );
-			if ( holder == null )	holder = data_map_sv.get( this );
-			if ( holder != null ) {
-				data = holder.data;
-			}
-		}
-		if ( data == null )	return null;
-		if ( holder.history_use ) {
-			if ( next_page.getID() == last_id ) {
-				getPageFactory().returnPage( next_page );
-				next_page = last_page;
-			}
-			else if ( next_page.isAllowHistoryAdd() ) {
-				Page	sub = data.hist.get( next_key ).getPage( next_page.getID() );
-				if ( sub != null ) {
-					getPageFactory().returnPage( next_page );
-					next_page = sub;
-				}
-			}
-		}
-		next_page.setRequestParameter( holder.request );
-		Page	view = SandBox.output( next_page, last_page, holder.request );
-		if ( view == null )	view = next_page;
-		next_page.setInitialized( false );
-		next_page.setRequestParameter( null );
-		return new Page[]{ next_page, view };
-	}
-
 	RequestParameter normalReq( HttpServletRequest req ) {
 		RequestParameter	ret = new RequestParameter();
 		Enumeration<?>	e = req.getParameterNames();
